<?php

namespace WpifyWoo\Abstracts;

use Spatie\ArrayToXml\ArrayToXml;
use Wpify\Core_4_0\Abstracts\AbstractComponent;
use WpifyWoo\Plugin;
use WpifyWoo\WooCommerceIntegration;

/**
 * Class AbstractModule
 * @package WpifyWoo\Abstracts
 * @property Plugin $plugin
 */
abstract class AbstractFeed extends AbstractComponent {
	private $feed_data = [];

	/**
	 * @param array $products
	 * @param int   $counter
	 *
	 * @return array
	 */
	abstract public function data( array $products, $counter = 0 ): array;

	abstract public function feed_name();

	public function get_xml_from_array( $data, $root_name = 'root' ) {
		$xml = new ArrayToXml( $data, $root_name );

		return $xml->prettify()->toXml();
	}

	public function generate_feed() {
		$this->save_feed( $this->get_feed_xml() );
	}

	public function get_feed_xml() {
		for ( $i = 1; $i < 1000000; $i ++ ) {
			$data = $this->get_data_for_page( $i );
			if ( ! $data ) {
				break;
			}

			$this->feed_data = array_merge( $this->feed_data, $data['data'] );
		}

		return $this->get_xml_from_array( $this->feed_data, $this->get_root_name() );
	}

	public function get_data_for_page( $page ) {
		$args     = [
			'limit'      => 100,
			'page'       => $page,
			'status'     => 'publish',
			'visibility' => 'visible',
		];

		$products = wc_get_products( $args );
		if ( empty( $products ) ) {
			return null;
		}

		$counter = count( $this->get_tmp_data() );

		return [
			'data'  => $this->data( $products, $counter ),
			'count' => count( $products ),
		];
	}

	public function add_tmp_data( array $data ) {
		$data = array_merge( $this->get_tmp_data(), $data );
		$this->save_tmp_data( $data );

		return $data;
	}

	public function delete_tmp_file(  ) {
		if (file_exists($this->get_tmp_file_path())) {
			unlink($this->get_tmp_file_path());
		}
	}

	public function save_tmp_data( array $data ) {
		if ( ! file_exists( $this->get_tmp_dir_path() ) ) {
			mkdir( $this->get_tmp_dir_path(), 0777, true );
		}

		return file_put_contents( $this->get_tmp_file_path(), json_encode( $data ) );
	}

	public function get_tmp_data() {
		if ( ! file_exists( $this->get_tmp_file_path() ) ) {
			return [];
		}

		$data = json_decode( file_get_contents( $this->get_tmp_file_path() ) ) ?: array();

		return $data ? json_decode( json_encode( $data ), true ) : array();
	}

	public function save_feed( $data ) {
		if ( ! file_exists( $this->get_dir_path() ) ) {
			mkdir( $this->get_dir_path(), 0777, true );
		}

		return file_put_contents( $this->get_xml_path(), $data );
	}

	public function get_root_name() {
		return 'root';
	}

	public function get_directory_name() {
		return 'xml';
	}

	public function get_file_name() {
		return sprintf( '%s_%s.xml', $this->feed_name(), get_current_blog_id() );
	}

	public function get_tmp_file_name() {
		return sprintf( '%s_%s_tmp.json', $this->feed_name(), get_current_blog_id() );
	}


	public function get_dir_path() {
		return trailingslashit( wp_upload_dir()['basedir'] ) . trailingslashit( $this->get_directory_name() );
	}

	public function get_dir_url() {
		return trailingslashit( wp_upload_dir()['baseurl'] ) . trailingslashit( $this->get_directory_name() );
	}

	public function get_tmp_dir_path() {
		return trailingslashit( wp_upload_dir()['basedir'] ) . trailingslashit( $this->get_directory_name() ) . 'tmp/';
	}

	public function get_xml_path() {
		return $this->get_dir_path() . $this->get_file_name();
	}

	public function get_xml_url() {
		return $this->get_dir_url() . $this->get_file_name();
	}

	public function get_tmp_file_path() {
		return $this->get_tmp_dir_path() . $this->get_tmp_file_name();
	}


}
