<?php

namespace WpifyWoo\Factories;

use Wpify\Core_4_0\Abstracts\AbstractCustomFieldsFactory;
use Wpify\Core_4_0\Abstracts\AbstractWooOrderModel;

class WooOrderFieldsFactory extends AbstractCustomFieldsFactory {

	/**
	 * Register hooks
	 *
	 * @return bool|void
	 */
	public function setup() {
		add_action( 'cmb2_admin_init', [ $this, 'register_metaboxes' ] );
	}

	/**
	 * Get a single field value
	 *
	 * @param AbstractWooOrderModel $order
	 * @param                       $field
	 *
	 * @return mixed
	 */
	public function get_field( $order, $field ) {
		return $order->get_wc_order()->get_meta( $field, true );
	}

	/**
	 * Register metaboxes
	 */
	public function register_metaboxes() {
		$cmb = new_cmb2_box(
			[
				'id'           => $this->get_entity_name() . '_metabox',
				'title'        => __( 'Details', 'wpify-woo-podani-online' ),
				'object_types' => [ $this->get_entity_name() ],
				'context'      => 'normal',
				'priority'     => 'high',
				'show_names'   => true,
			]
		);

		foreach ( $this->get_custom_fields() as $field ) {
			if ( ! is_array( $field ) ) {
				continue;
			}
			$cmb->add_field( $field );
		}
	}

	/**
	 * Save custom field value
	 *
	 * @param AbstractWooOrderModel $order
	 * @param                       $field
	 * @param                       $value
	 *
	 * @return bool|int
	 */
	public function save_field( $order, $field, $value ) {
		return $order->get_wc_order()->update( $field, true );
	}
}
