<?php

namespace WpifyWoo\Modules\XmlFeedHeureka;

use WpifyWoo\Abstracts\AbstractModule;

class XmlFeedHeurekaModule extends AbstractModule {
	/**
	 * @var Feed
	 */
	private $feed;

	private $temp_categories = [];

	public function __construct( Feed $feed ) {
		parent::__construct();
		$this->feed = $feed;
	}

	public function setup() {
		add_filter( 'wpify_woo_settings_' . $this->id(), array( $this, 'settings' ) );
		add_action( 'admin_init', [ $this, 'handle_actions' ] );
		add_filter( 'woocommerce_product_data_tabs', [ $this, 'add_product_tabs' ] );
		add_action( 'woocommerce_product_data_panels', [ $this, 'add_product_tabs_content' ] );
		add_action( 'woocommerce_process_product_meta', [ $this, 'save_custom_fields' ] );
		add_action( 'woocommerce_product_after_variable_attributes', [ $this, 'add_custom_variations_fields' ], 10, 3 );
		add_action( 'woocommerce_save_product_variation', [ $this, 'save_custom_variation_fields' ], 10, 2 );
	}

	function add_product_tabs( $tabs ) {
		$tabs['additional_info'] = [
				'label'    => __( 'Heureka XML', 'wpify-woo' ),
				'target'   => 'wpify_woo_heureka_xml',
				'priority' => 200,
		];

		return $tabs;
	}

	public function add_product_tabs_content() { ?>
		<div id="wpify_woo_heureka_xml" class="panel woocommerce_options_panel hidden"><?php

		woocommerce_wp_text_input( [
				'id'    => '_wpify_woo_heureka_product_name',
				'label' => __( 'Heureka Product name', 'wpify-woo' ),
		] );
		woocommerce_wp_text_input( [
				'id'    => '_wpify_woo_heureka_product',
				'label' => __( 'Heureka Product', 'wpify-woo' ),
		] );
		woocommerce_wp_text_input( [
				'id'    => '_wpify_woo_heureka_category',
				'label' => __( 'Heureka Category', 'wpify-woo' ),
		] );

		?></div><?php
	}

	public function add_custom_variations_fields( $loop, $variation_data, $variation ) {
		echo '<div class="options_group form-row form-row-full">';

		woocommerce_wp_text_input(
				array(
						'id'    => '_wpify_woo_heureka_product_name[' . $variation->ID . ']',
						'label' => __( 'Heureka Product name', 'wpify-woo' ),
						'value' => get_post_meta( $variation->ID, '_wpify_woo_heureka_product_name', true ),
				)
		);
		woocommerce_wp_text_input(
				array(
						'id'    => '_wpify_woo_heureka_product[' . $variation->ID . ']',
						'label' => __( 'Heureka Product', 'wpify-woo' ),
						'value' => get_post_meta( $variation->ID, '_wpify_woo_heureka_product', true ),
				)
		);
		woocommerce_wp_text_input(
				array(
						'id'    => '_wpify_woo_heureka_category[' . $variation->ID . ']',
						'label' => __( 'Heureka Category', 'wpify-woo' ),
						'value' => get_post_meta( $variation->ID, '_wpify_woo_heureka_category', true ),
				)
		);
		echo '</div>';
	}

	function save_custom_variation_fields( $post_id ) {
		update_post_meta( $post_id, '_wpify_woo_heureka_product_name', sanitize_text_field( $_POST['_wpify_woo_heureka_product_name'][ $post_id ] ) );
		update_post_meta( $post_id, '_wpify_woo_heureka_product', sanitize_text_field( $_POST['_wpify_woo_heureka_product'][ $post_id ] ) );
		update_post_meta( $post_id, '_wpify_woo_heureka_category', sanitize_text_field( $_POST['_wpify_woo_heureka_category'][ $post_id ] ) );
	}

	public function save_custom_fields( $post_id ) {
		$product = wc_get_product( $post_id );
		$product->update_meta_data( '_wpify_woo_heureka_product_name', sanitize_text_field( $_POST['_wpify_woo_heureka_product_name'] ) );
		$product->update_meta_data( '_wpify_woo_heureka_product', sanitize_text_field( $_POST['_wpify_woo_heureka_product'] ) );
		$product->update_meta_data( '_wpify_woo_heureka_category', sanitize_text_field( $_POST['_wpify_woo_heureka_category'] ) );
		$product->save();
	}

	/**
	 * Set the module ID.
	 * @return string
	 */
	public function id(): string {
		return 'xml_feed_heureka';
	}

	/**
	 * Set the module ID.
	 * @return string
	 */
	public function name() {
		return __( 'XML Feed Heureka', 'wpify-woo' );
	}

	/**
	 * Add settings
	 * @return array[] Settings.
	 */
	public function settings(): array {
		$heureka_categories = $this->get_heureka_categories_select();

		if ( empty( $heureka_categories ) ) {
			$this->update_heureka_categories();
		}

		$delivery_methods = $this->get_heureka_delivery_methods_select();

		$settings = array(
				array(
						'id'            => 'delivery',
						'type'          => 'text',
						'label'         => __( 'Delivery time', 'wpify-woo' ),
						'desc'          => __( 'Enter 0 for instock, 1-3 for 3 days, 4-7 for one week, 8-14 for two weeks, 15-30 for one month, 31 and more for month and more.', 'wpify-woo' ),
						'default_value' => '0',
				),
				array(
						'id'            => 'delivery_out_of_stock',
						'type'          => 'text',
						'label'         => __( 'Delivery time for out of stock items', 'wpify-woo' ),
						'desc'          => __( 'Enter 0 for instock, 1-3 for 3 days, 4-7 for one week, 8-14 for two weeks, 15-30 for one month, 31 and more for month and more.', 'wpify-woo' ),
						'default_value' => '0',
				),
				array(
						'id'    => 'exclude_outofstock',
						'type'  => 'switch',
						'label' => __( 'Exclude out of stock items', 'wpify-woo' ),
						'desc'  => __( 'Check to exclude out of stock items.', 'wpify-woo' ),
				),
				array(
						'id'    => 'item_id_custom_field',
						'type'  => 'text',
						'label' => __( 'ITEM_ID custom field', 'wpify-woo' ),
						'desc'  => __( 'Product ID is used as default value for ITEM_ID. Enter custom field key if you want to use custom field value instead.', 'wpify-woo' ),
				),
				array(
						'id'    => 'ean_custom_field',
						'type'  => 'text',
						'label' => __( 'EAN custom field', 'wpify-woo' ),
						'desc'  => __( 'SKU is used as default value for EAN. Enter custom field key if you want to use custom field value instead.', 'wpify-woo' ),
				),
		);

		$settings[] = array(
				'id'    => 'delivery_methods_title',
				'type'  => 'title',
				'label' => __( 'Delivery methods', 'wpify-woo' ),
				'desc'  => __( 'Select the delivery methods and prices', 'wpify-woo' ),
		);

		$settings[] = array(
				'id'      => 'delivery_methods',
				'type'    => 'group',
				'label'   => __( 'Delivery methods', 'wpify-woo' ),
				'multi'   => true,
				'min'     => 0,
				'buttons' => array(
						'add'    => __( 'Add method', 'wpify-woo' ),
						'remove' => __( 'Remove method', 'wpify-woo' ),
				),
				'items'   => array(
						array(
								'id'      => 'method',
								'type'    => 'select',
								'label'   => __( 'Delivery method', 'wpify-woo' ),
								'desc'    => __( 'Select delivery method', 'wpify-woo' ),
								'options' => $delivery_methods,
						),
						array(
								'id'    => 'price',
								'type'  => 'text',
								'label' => __( 'Price', 'wpify-woo' ),
								'desc'  => __( 'Enter price for delivery.', 'wpify-woo' ),
						),
						array(
								'id'    => 'price_cod',
								'type'  => 'text',
								'label' => __( 'Price COD', 'wpify-woo' ),
								'desc'  => __( 'Enter price for delivery with COD.', 'wpify-woo' ),
						),
				),
		);

		$settings[] = array(
				'id'    => 'map_categories_title',
				'type'  => 'title',
				'label' => __( 'Map categories', 'wpify-woo' ),
				'desc'  => __( 'Map WooCommerce categories to Heureka categories', 'wpify-woo' ),
		);

		$settings[] = array(
				'id'    => 'update_categories_button',
				'type'  => 'button',
				'desc'  => __( 'Click to update the Heureka categories.', 'wpify-woo' ),
				'label' => __( 'Update Heureka categories', 'wpify-woo' ),
				'url'   => add_query_arg( [ 'wpify-woo-action' => 'update-heureka-categories' ], $this->get_settings_url() ),
		);

		foreach ( get_terms( [ 'taxonomy' => 'product_cat', 'hide_empty' => false ] ) as $category ) {
			$settings[] = [
					'id'      => 'heureka_category_' . $category->term_id,
					'label'   => sprintf( __( 'Heureka category for %s', 'wpify-woo' ), $category->name ),
					'type'    => 'select',
					'options' => $heureka_categories,
			];
		}

		$settings[] = array(
				'id'             => 'generate_button',
				'type'           => 'generate_feed',
				'desc'           => sprintf(
						__( 'Click to regenerate feed. Make sure to save the settings before generating the feed.<br/>The feed will be available at <a href="%1$s" target="_blank"><code style="-webkit-user-select: all;user-select: all;">%1$s</code></a>.<br/>You can also setup cron job to <code style="-webkit-user-select: all;user-select: all;">%2$s</code> to regenerate the feed automatically.',
								'wpify-woo' ),
						$this->feed->get_xml_url(),
						$this->plugin->get_api_manager()->get_rest_url() . '/feed/generate/heureka'
				),
				'label'          => __( 'Generate feed', 'wpify-woo' ),
				'feed_chunk_url' => $this->plugin->get_api_manager()->get_rest_url() . '/feed/chunk-generate/heureka',
		);

		return $settings;
	}

	public function get_heureka_categories_select() {
		$select = [];
		foreach ( $this->get_heureka_categories() as $id => $category ) {
			$select[] = [
					'label' => $category['category_fullname'] ?: $category['category_name'],
					'value' => $id,
			];
		}

		return $select;
	}

	public function get_heureka_categories() {
		return get_option( 'wpify_woo_heureka_xml_categories', [] );
	}

	public function update_heureka_categories() {
		if ( ( $response_xml_data = file_get_contents( 'https://www.heureka.cz/direct/xml-export/shops/heureka-sekce.xml' ) ) === false ) {
			//echo "Error fetching XML\n";
		} else {
			$feed = simplexml_load_string( $response_xml_data );
			foreach ( $feed->CATEGORY as $first_level ) {
				$id                                                = (string) $first_level->CATEGORY_ID;
				$name                                              = (string) $first_level->CATEGORY_NAME;
				$this->temp_categories[ $id ]['category_id']       = $id;
				$this->temp_categories[ $id ]['category_name']     = $name;
				$this->temp_categories[ $id ]['category_fullname'] = '';
				$this->build_categories( $first_level->CATEGORY, $id );
			}
			update_option( 'wpify_woo_heureka_xml_categories', $this->temp_categories );
		}
	}

	public function build_categories( $data, $category_id ) {
		if ( ! empty( $data ) ) {
			foreach ( $data as $item ) {
				$item_id       = (string) $item->CATEGORY_ID;
				$item_name     = (string) $item->CATEGORY_NAME;
				$item_fullname = (string) $item->CATEGORY_FULLNAME;

				if ( ! empty( $item_fullname ) ) {
					$this->temp_categories[ $item_id ]['category_id']       = $item_id;
					$this->temp_categories[ $item_id ]['category_name']     = $item_name;
					$this->temp_categories[ $item_id ]['category_fullname'] = $item_fullname;
				}

				$this->build_categories( $item->CATEGORY, $category_id );
			}
		}
	}

	public function get_heureka_delivery_methods_select() {
		$select = [];

		foreach ( $this->get_heureka_delivery_methods() as $id => $label ) {
			$select[] = [
					'value' => $id,
					'label' => $label,
			];
		}

		return $select;
	}

	public function get_heureka_delivery_methods() {
		return [
				'CESKA_POSTA'                    => 'Česká pošta - Balík Do ruky',
				'CESKA_POSTA_NAPOSTU_DEPOTAPI'   => 'Česká pošta - Balík Na poštu',
				'CESKA_POSTA_DOPORUCENA_ZASILKA' => 'Česká pošta - Doporučená zásilka',
				'CSAD_LOGISTIK_OSTRAVA'          => 'ČSAD Logistik Ostrava',
				'DPD'                            => 'DPD (nejedná se o DPD ParcelShop)',
				'DHL'                            => 'DHL',
				'DSV'                            => 'DSV',
				'FOFR'                           => 'FOFR',
				'GEBRUDER_WEISS'                 => 'Gebrüder Weiss',
				'GEIS'                           => 'Geis (nejedná se o Geis Point)',
				'GLS'                            => 'GLS',
				'HDS'                            => 'HDS',
				'PPL'                            => 'PPL',
				'SEEGMULLER'                     => 'Seegmuller',
				'TNT'                            => 'TNT',
				'UPS'                            => 'UPS',
				'FedEX'                          => 'FedEX',
				'RABEN_LOGISTICS'                => 'Raben Logistics',
				'ZASILKOVNA'                     => 'Zásilkovna',
				'DPD_PICKUP'                     => 'DPD Pickup',
				'BALIKOVNA_DEPOTAPI'             => 'Balíkovna',
				'VLASTNI_PREPRAVA'               => 'Vlastní přeprava',
				'WEDO'                           => 'WeDo (IN TIME & Uloženka)',

		];
	}

	public function handle_actions() {
		if ( isset( $_GET['wpify-woo-action'] ) && 'update-heureka-categories' === $_GET['wpify-woo-action'] ) {
			$this->update_heureka_categories();
		}
	}

	/**
	 * @return Feed
	 */
	public function get_feed(): Feed {
		return $this->feed;
	}
}
